# -*- coding: UTF-8 -*-
from modules.repository import Repository
from modules.scheduler import task_scheduler
import json
from modules.tools import DatetimeEncoder


class PlatformRepository(Repository):
    def __init__(self):
        Repository.__init__(self, db="fashion_product")

    def get_platform_product(self, product_id):
        sql = """
        SELECT * FROM t_product WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            return cursor.fetchone()

    def get_product_sku_images(self, product_id):
        sql = """
        SELECT a.* FROM fashion_product.t_product_sku a 
        WHERE a.product_id=%s 
        AND a.sku_images IS NOT NULL 
        AND a.sku_images <> 'null' 
        AND trim(a.sku_images) <> '' 
        """

        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            sku = cursor.fetchone()
        try:
            return json.loads(sku["sku_images"] or '[]') if sku else None
        except:
            return None

    def save_product_images(self, product_id, cover, image_urls):
        product = self.get_platform_product(product_id)
        if not product:
            print("product no found:", product_id)
            return

        if product["cover"] and product["cover"] != 'null':
            print("skip replace exists cover:\n%s\n%s" % (product["cover"], cover))
            cover = product["cover"]

        # 已经上架的商品不修改图片
        sql = """
        UPDATE fashion_product.t_product a,
         fashion_product.t_product_sku b
        SET 
         a.cover = %s,
         b.sku_cover = %s,
         b.sku_images = %s,
         b.remark = 'crawler'
        WHERE
            a.product_id = b.product_id
        AND a.product_id = %s
        """
        print("set product images[0]:", image_urls[0])
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (cover, cover, json.dumps(image_urls), product_id))
        self.connection.commit()


class BuyerRepository(Repository):
    def __init__(self):
        Repository.__init__(self, db="fashion_buyer")

    def create_picture(self, picture):
        sql = """
               INSERT INTO `fashion_buyer`.`t_picture` 
               (`picture_id`,`name`, `cover`, `cover_type`, `description`, `type`, `date`, `tags`, `image_urls`, `date_added`) 
               VALUES 
               (%(picture_id)s, %(picture_name)s, %(cover)s, '0', %(description)s , '3',now(), %(tags)s, %(image_urls)s, now());
               """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, picture)
            lastrowid = cursor.lastrowid
        self.connection.commit()
        return lastrowid

    def check_picture_source(self, picture):
        sql = """
        SELECT * FROM fashion_buyer.t_picture_source WHERE source_id=%s AND source_type=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (picture["source_id"], picture["source_type"]))
            return cursor.fetchone()

    def create_picture_source(self, picture):
        sql = """
        INSERT INTO `fashion_buyer`.`t_picture_source` 
        (`source_id`, `source_type`, `url`, `date_added`) 
        VALUES 
        (%(source_id)s, %(source_type)s, %(url)s, now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, picture)
            lastrowid = cursor.lastrowid
        self.connection.commit()
        return lastrowid

    def update_picture_source(self, picture):
        sql = """
        UPDATE `fashion_buyer`.`t_picture_source` 
        SET 
        `source_id`=%(source_id)s, 
        `source_type`=%(source_type)s, 
        `url`=%(url)s,  
        `date_modified`=now() WHERE (`picture_id`=%(picture_id)s);
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, picture)
        self.connection.commit()

    def update_picture(self, picture):
        print "update picture:", picture
        sql = """
        UPDATE `fashion_buyer`.`t_picture` SET 
        `name`=%(picture_name)s, 
        `cover`=%(cover)s, 
        `cover_type`='0', 
        `description`=%(description)s, 
        `type`='3', 
        `image_urls`=%(image_urls)s, 
        `date_modified`=now() 
        WHERE (`picture_id`=%(picture_id)s);
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, picture)
        self.connection.commit()

    def check_picture(self, picture):
        sql = """
        SELECT * FROM fashion_buyer.t_picture WHERE `picture_id` = %s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, picture["picture_id"])
            return cursor.fetchone()

    def save_picture(self, picture):
        # FIXME: 应该创建picture_source 再创建picture,这样可以支持name相同多个图片来源
        picture_source = self.check_picture_source(picture)
        if picture_source:
            picture["picture_id"] = picture_source["picture_id"]
            self.update_picture_source(picture)
        else:
            picture["picture_id"] = self.create_picture_source(picture)

        if self.check_picture(picture):
            self.update_picture(picture)
        else:
            self.create_picture(picture)

    def get_picture_by_source(self, source_id, source_type):
        sql = """
        SELECT
            b.*
        FROM
            fashion_buyer.t_picture b,
            fashion_buyer.t_picture_source a
        WHERE
            a.picture_id = b.picture_id
        AND a.source_id = %s
        AND a.source_type = %s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (source_id, source_type))
            return cursor.fetchone()

    def add_picture_tags(self, source_id, source_type, tags):
        if not tags:
            return

        picture = self.get_picture_by_source(source_id, source_type)
        if not picture:
            print("picture source no found:", source_id, source_type)
            return

        exists_tags = set(json.loads(picture["tags"]) or [])
        new_tags = list(set(tags) | exists_tags)

        sql = """
        UPDATE fashion_buyer.t_picture SET tags=%s WHERE picture_id=%s
        """
        print("add tags for picture:", picture["picture_id"], new_tags)
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (json.dumps(new_tags).decode("unicode-escape"), picture["picture_id"]))
        self.connection.commit()


class TaskRepository(Repository):
    KEY_COVER_TASKS = "platform:cover:tasks"
    KEY_PICTURE_TASKS = "platform:picture:tasks"
    KEY_PRODUCT_TASKS = "platform:product:tasks"
    KEY_MONITOR_TASKS = "platform:monitor:tasks"

    def __init__(self):
        Repository.__init__(self, db="spider")

    def get_picture_task_length(self):
        return task_scheduler.length(self.KEY_PICTURE_TASKS)

    def create_picture_tasks(self):
        task_length = self.get_picture_task_length()
        if task_length > 0:
            print ("no create tasks, has task in queue:", task_length)
            return

        sql = """
        SELECT url, source_id, source_type FROM fashion_buyer.t_picture_source WHERE date(date_added) = date(now())
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            product_sources = cursor.fetchall()

        for source in product_sources:
            task_scheduler.push(self.KEY_PICTURE_TASKS, json.dumps(source))

    def get_picture_task(self):
        picture_task = task_scheduler.pop(self.KEY_PICTURE_TASKS)
        return json.loads(picture_task) if picture_task else None

    def get_product_task_length(self):
        return task_scheduler.length(self.KEY_PRODUCT_TASKS)

    def set_product_task_status(self, product_id, task_status, cover_url):
        sql = """
        UPDATE spider.t_product_task SET `status`=%s, cover_url=%s WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (task_status, cover_url, product_id))
        self.connection.commit()

    def create_product_tasks(self):
        task_length = self.get_product_task_length()
        if task_length > 0:
            print ("no create tasks, has task in queue:", task_length)
            return

        sql = """
        SELECT
            b.*, c.source_type,
            c.source_id
        FROM
            spider.t_product_task a,
            fashion_product.t_product b,
            spider.t_product_source c
        WHERE
            a.product_id = b.product_id
        AND c.product_id = a.product_id
        AND a.`status` = 0
        """

        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            download_tasks = cursor.fetchall()

        for product_task in download_tasks:
            print("create product task:", product_task)
            task_scheduler.push(self.KEY_PRODUCT_TASKS, json.dumps(product_task))

    def get_product_task(self):
        product_task = task_scheduler.pop(self.KEY_PRODUCT_TASKS)
        return json.loads(product_task) if product_task else None

    def add_ftp_cover_task(self, cover_task):
        task_scheduler.push(self.KEY_COVER_TASKS, json.dumps(cover_task))

    def get_ftp_cover_task(self):
        cover_task = task_scheduler.pop(self.KEY_COVER_TASKS)
        return json.loads(cover_task) if cover_task else None

    def get_monitor_task_length(self):
        return task_scheduler.length(self.KEY_MONITOR_TASKS)

    def get_monitor_task(self):
        monitor_task = task_scheduler.pop(self.KEY_MONITOR_TASKS)
        return json.loads(monitor_task) if monitor_task else None

    def create_monitor_tasks(self, source_type):
        # 默认是farfetch
        source_type = source_type if source_type else 1
        task_length = self.get_monitor_task_length()
        if task_length > 0:
            print("have monitor task in queue:", task_length)
            return

        print("start create monitor tasks...")
        sql = """
        SELECT
            c.url,
            b.*
        FROM
            fashion_product.t_product a,
            spider.t_product_source b,
            spider.t_catalog c
        WHERE
            a.product_id = b.product_id
        AND a.product_status = 1
        AND b.source_id = c.source_id
        AND b.source_type = c.source_type
        AND b.source_type = %s
        """

        with self.connection.cursor() as cursor:
            cursor.execute(sql % source_type)
            monitor_tasks = cursor.fetchall()

        for monitor_task in monitor_tasks:
            print("create monitor task:", monitor_task)
            task_scheduler.push(self.KEY_MONITOR_TASKS, json.dumps(monitor_task, cls=DatetimeEncoder))


class CatalogRepository(Repository):
    def __init__(self):
        Repository.__init__(self, db="spider")

    def check_catalog(self, catalog):
        sql = "SELECT * FROM t_catalog WHERE url=%s"
        with self.connection.cursor() as cursor:
            cursor.execute(sql, catalog["url"])
            return cursor.fetchone()

    def update_catalog(self, catalog):
        print "update catalog[%s]:" % catalog["product_name"], catalog["url"]
        sql = """
        UPDATE `spider`.`t_catalog` SET 
        `url`=%(url)s, 
        `name`=%(product_name)s, 
        `source_type`=%(source_type)s, 
        `source_id`=%(source_id)s, 
        `thumbnail`=%(thumbnail)s,
        `designer_name`=%(designer_name)s,
        `quantity`=%(quantity)s,
        `season`=%(season)s,
        `language_id`=%(language_id)s, 
        `date_modified`=now() 
        WHERE (`catalog_id`=%(catalog_id)s)
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, catalog)
        self.connection.commit()

    def save_catalog_context(self, catalog):
        sql = """
        REPLACE INTO `spider`.`t_catalog_context` (
            `catalog_id`,
            `context`,
            `date_added`
        )VALUES(
            %(catalog_id)s,
            %(catalog_context)s,
            now()
        )
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, catalog)
        self.connection.commit()

    def create_catalog(self, catalog):
        print "create catalog[%s]:" % catalog["product_name"], catalog["url"]
        # 同一个商品会出现在多个店铺直接replace into
        sql = """
        REPLACE INTO `spider`.`t_catalog` 
        (`url`, `name`, `source_type`, `source_id`,`language_id`, `thumbnail`,`designer_name`,`season`,`quantity`, `status`, `date_added`) 
        VALUES 
        (%(url)s,%(product_name)s,%(source_type)s,%(source_id)s,%(language_id)s,%(thumbnail)s,%(designer_name)s,%(season)s,%(quantity)s,%(catalog_status)s,now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, catalog)
            lastrowid = cursor.lastrowid
        self.connection.commit()
        return lastrowid

    def save_catalog(self, catalog):
        exists_catalog = self.check_catalog(catalog)
        if exists_catalog:
            catalog["catalog_id"] = exists_catalog["catalog_id"]
            self.update_catalog(catalog)
        else:
            catalog["catalog_id"] = self.create_catalog(catalog)
        # 保存商品店铺信息
        if catalog.get("store_id"):
            self.save_source_store(catalog)

        # 保存目录context
        self.save_catalog_context(catalog)

    def save_source_store(self, catalog):
        sql = """
        REPLACE INTO `spider`.`t_source_store` 
        (`url`, `source_id`, `source_type`, `store_id`, `date_added`) 
        VALUES (%(url)s, %(source_id)s, %(source_type)s, %(store_id)s, now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, catalog)
        self.connection.commit()

    def set_catalog_status(self, catalog, status):
        sql = """
        UPDATE t_catalog SET status=%s WHERE url=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (status, catalog["url"]))
        self.connection.commit()

    def save_product_color(self, color):
        if self.get_product_color(color["color_code"], 1):
            return

        sql = """
        INSERT INTO `spider`.`t_color` 
        (`color_code`, `source_type`, `name`, `description`, `date_added`) 
        VALUES 
        (%(color_code)s, %(source_type)s, %(color_name)s, %(description)s, now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, color)
        self.connection.commit()

    def get_product_color(self, color_code, source_type):
        sql = """
        SELECT * FROM spider.t_color WHERE color_code=%s AND source_type=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (color_code, source_type))
            return cursor.fetchone()

    def check_product_cache(self, catalog):
        sql = """
        select * from spider.t_product_cache where source_id=%s and source_type=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (catalog["source_id"], catalog["source_type"]))
            return cursor.fetchone()

    def get_product_cache(self, catalog):
        sql = """
        select * from spider.t_product_cache where 
        source_id=%(source_id)s 
        and source_type=%(source_type)s
        and date_modified > DATE_SUB(NOW(),INTERVAL  15 MINUTE)
        """

        with self.connection.cursor() as cursor:
            cursor.execute(sql, catalog)
            product_cache = cursor.fetchone()
            if not product_cache:
                return None
        product_context = product_cache['context']
        return json.loads(product_context) if product_context else None

    def update_product_cache(self, product_cache):
        sql = """
        UPDATE `spider`.`t_product_cache` 
        SET 
        `source_id`=%(source_id)s, 
        `source_type`=%(source_type)s, 
        `context`=%(product_context)s, 
        `date_modified`=now() 
        WHERE 
        `source_id`=%(source_id)s 
        AND `source_type`=%(source_type)s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_cache)
        self.connection.commit()

    def create_product_cache(self, product_cache):
        sql = """
        INSERT INTO `spider`.`t_product_cache` 
        (`source_id`, `source_type`, `context`, `date_added`) 
        VALUES 
        (%(source_id)s, %(source_type)s, %(product_context)s, now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_cache)
        self.connection.commit()

    def save_product_cache(self, catalog, product):
        product_cache = {
            "product_context": json.dumps(product),
            "source_id": catalog["source_id"],
            "source_type": catalog["source_type"]
        }
        if self.check_product_cache(catalog):
            self.update_product_cache(product_cache)
        else:
            self.create_product_cache(product_cache)


class MonitorRepository(Repository):
    def __init__(self):
        Repository.__init__(self)

    def set_platform_product_status(self, product_id, product_status):
        print("[%s]set product status=%s" % (product_id, product_status))
        sql = """
        UPDATE fashion_product.t_product 
        SET product_status=%s,
        modify_time=unix_timestamp(),
        editor_name='monitor' 
        WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (product_status, product_id))
            self.connection.commit()
            cursor.close()

    def get_platform_product(self, product_id):
        sql = """
        SELECT * FROM fashion_product.t_product WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            product = cursor.fetchone()

        if not product:
            return product

        sql = """
           SELECT * FROM fashion_product.t_product_sku WHERE product_id=%s AND sku_status='ON'
           """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            product["skus"] = cursor.fetchall()

        return product

    def set_platform_product_sizes_status(self, product_id, size, sku_status):
        sql = """
        UPDATE fashion_product.t_product_sku SET 
            sku_status=%s,
            modify_time=unix_timestamp(),
            editor_name='monitor' 
        WHERE product_id=%s AND size=%s
        """
        print("[%s]set product size:%s status=%s" % (product_id, size, sku_status))
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (sku_status, product_id, size))
            self.connection.commit()
            cursor.close()

        sql = """
        REPLACE INTO `spider`.`t_monitor_sku` 
            (`product_id`, `size`, `status`, `date_modified`) 
        VALUES 
            (%s, %s, %s, now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (product_id, size, 0 if "OFF" == sku_status else 0))
            self.connection.commit()
            cursor.close()

    def set_platform_product_size_quantity(self, product_id, size, quantity):
        sql = """
        UPDATE fashion_product.t_product_sku 
        SET quantity=%s,modify_time=unix_timestamp(),editor_name='monitor' 
        WHERE product_id=%s AND size=%s
        """
        print("[%s]set product sku:%s quantity=%s" % (product_id, size, quantity))
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (quantity, product_id, size))
            self.connection.commit()
            cursor.close()

    def update_platform_product_time(self, product_id):
        sql = """
        update fashion_product.t_product set modify_time=now() where product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            self.connection.commit()
            cursor.close()

    def update_platform_product_sku(self, product_id, sku):
        sql = """
        UPDATE fashion_product.t_product_sku 
        SET quantity=%s,
        original_price=%s,
        supply_price=%s,
        modify_time=unix_timestamp(),
        editor_name='monitor' 
        WHERE product_sku_id=%s
        """
        print("[%s]>>>update product sku<<<<:" % product_id, sku)
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (
                sku['quantity'],
                sku['original_price'],
                sku['supply_price'],
                sku['product_sku_id']
            ))
            self.connection.commit()
            cursor.close()

    def get_last_monitored_time(self, product_id):
        sql = "SELECT * FROM `spider`.t_monitor_product WHERE product_id=%s"
        with self.connection.cursor() as cursor:
            cursor.execute(sql, product_id)
            return cursor.fetchone()

    def update_monitored_time(self, monitor_task, status):
        sql = """
        REPLACE INTO `spider`.`t_monitor_product` 
        (`product_id`, `source_type`, `source_id`, `status`, `date_modified`) 
        VALUES 
        (%s, %s, %s, %s, now());
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (
                monitor_task["product_id"],
                monitor_task["source_type"],
                monitor_task["source_id"],
                status))
            self.connection.commit()


class Dao:
    def __init__(self):
        self.catalog = None
        self.create_repositories()

    def create_repositories(self):
        self.catalog = CatalogRepository()
        self.platform = PlatformRepository()
        self.monitor = MonitorRepository()
        self.task = TaskRepository()

    def reset(self):
        self.create_repositories()


dao = Dao()



import os
from mozia.crawler.repository import dao
from modules.repository import repositories


def find_files(dir):
    return ["%s/%s" % (dir, local_file) for local_file in os.listdir(dir)]
    # print("put ftp file:", local_file)
    # save_file_to_ftp(remote_path, local_path)


def find_covers():
    covers = []
    cover_dir = "F:/home/scan"
    for subdir in os.listdir(cover_dir):
        covers += find_files("%s/%s" % (cover_dir, subdir))

    return covers


def create_formatted_covers():
    for cover in find_covers():
        print(cover)


if __name__ == "__main__":
    # find_covers()
    repositories.connect(host='10.26.235.6', port=3306)
    create_formatted_covers()



# -*- coding: UTF-8 -*-
from ftplib import FTP

__ftp_client__ = None


def get_ftp_client():
    global __ftp_client__
    if not __ftp_client__:
        __ftp_client__ = FTP()
        __ftp_client__.connect(host="172.16.8.151")
        __ftp_client__.login("ftp", "ftp@123456")
        __ftp_client__.cwd("off-images")

    return __ftp_client__



import os
from ftp_client import get_ftp_client


def save_file_to_ftp(remote_path, local_path):
    ftp_client = get_ftp_client()

    buffer_size = 1024
    fp = open(local_path, 'rb')
    ftp_client.storbinary('STOR ' + remote_path, fp, buffer_size)
    ftp_client.set_debuglevel(0)
    fp.close()


if __name__ == "__main__":
    cover_dir = "G:/spider/ready/cover"
    for local_file in os.listdir(cover_dir):
        remote_path = "cover" + "/" + os.path.basename(local_file)
        local_path = cover_dir + "/" + local_file
        print "put ftp file:", remote_path, local_path
        save_file_to_ftp(remote_path, local_path)



from rediscluster import StrictRedisCluster


class RedisTaskScheduler:
    def __init__(self):
        self.redis = None

    def connect(self, nodes=[{"host": "172.16.8.147", "port": "6379"}]):
        print "connect to redis cluster:", nodes
        self.redis = StrictRedisCluster(startup_nodes=nodes, decode_responses=True)

    def push(self, name, task):
        # print "push task:" + task
        self.redis.lpush(name, task)

    def pop(self, name):
        return self.redis.rpop(name)

    def length(self, name):
        return self.redis.llen(name)

    def get(self, name):
        return self.redis.get(name)


if __name__ == "__main__":
    print ""



import os
from modules.repository import Repository, repositories


class RepairRepository(Repository):
    def __init__(self):
        Repository.__init__(self)

    def get_task_products(self):
        sql = """
        SELECT
        a.cover,
        a.product_id,
        b.status,
        c.sku_images,
        c.product_id AS id2
        FROM
        fashion_product.t_product a,
        spider.t_product_task b,
        fashion_product.t_product_sku c
        WHERE
        a.product_id = b.product_id
        AND b.`status` IN (1)
        AND c.product_id = a.product_id
        GROUP BY
        a.product_id
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    def set_product_task_url(self, product_id, url):
        sql = """
        UPDATE spider.t_product_task SET cover_url=%s WHERE product_id=%s
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (url, product_id))
        self.connection.commit()


repair = RepairRepository()

repositories.connect(host="10.26.235.6", port=3066)

for product in repair.get_task_products():
    cover_url = "%s/00.jpg" % os.path.dirname(product['cover'])
    print(cover_url, product["status"])
    repair.set_product_task_url(product["product_id"], cover_url)



# -*- coding: UTF-8 -*-
import requests
import urllib


def download(url):
    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/59.0.3071.115 Safari/537.36"
    }
    try:
        page = requests.get(url, headers=headers)
        return page.content
    except Exception as e:
        print(repr(e))
        return download(url)


# 内容不全重新下载
def download_image(url, filename):
    try:
        urllib.urlretrieve(url, filename)
    except urllib.ContentTooShortError:
        print 'Network conditions is not good.Reloading.', url
        download_image(url, filename)



# -*- coding: UTF-8 -*-
import pymysql
import json
import time


class DesignerSynchronizor:
    def __init__(self):
        self.connection = pymysql.connect(host='172.16.8.147',
                                          port=3306,
                                          user='dba',
                                          passwd='123456',
                                          db='fashion_product',
                                          cursorclass=pymysql.cursors.DictCursor,
                                          charset='utf8mb4')

    def start(self):
        sql = """
        SELECT DISTINCT (TRIM(designer)) AS designer_name FROM  test.t_intra_mirror
        WHERE TRIM(designer) NOT IN (  SELECT `name` FROM fashion_product.t_designer b )
        """
        with self.connection.cursor() as cursor:
            cursor.execute(sql)
            for designer in cursor.fetchall():
                self.create_designer(designer)

    def create_designer(self, designer):
        sql = """
        INSERT INTO `fashion_product`.`t_designer` (
            `name`,`first_char`,`desc`, `create_time`,`deleted`) VALUES (%s,%s,'intra mirror',UNIX_TIMESTAMP(),'0')
        """
        print "save designer", designer
        designer_name = designer["designer_name"]
        first_char = designer_name[1].upper()
        with self.connection.cursor() as cursor:
            cursor.execute(sql, (designer_name, first_char))
            self.connection.commit()


"""
DELETE FROM t_shop_product_sku WHERE sn LIKE "intramirror%";
DELETE FROM t_shop_product WHERE sn LIKE "intramirror%";
DELETE FROM t_shop_stock WHERE shop_product_sku_id IN (SELECT shop_product_sku_id FROM t_shop_product_sku WHERE sn LIKE "intramirror%");
"""


class ProductSynchronizer:
    def __init__(self):
        # 连接平台数据库
        self.platform_connection = pymysql.connect(host='10.26.235.6',
                                                   port=3066,
                                                   user='dba',
                                                   passwd='123456',
                                                   db='fashion_product',
                                                   cursorclass=pymysql.cursors.DictCursor,
                                                   charset='utf8mb4')

        # 连接intramirror数据库
        self.intramirror_connection = pymysql.connect(host='172.16.8.147',
                                                      port=3306,
                                                      user='dba',
                                                      passwd='123456',
                                                      db='test',
                                                      cursorclass=pymysql.cursors.DictCursor,
                                                      charset='utf8mb4')
        self.designers = {}
        self.load_designers()
        self.created_counter = 0
        self.updated_counter = 0

    # 加载品牌信息
    def load_designers(self):
        sql = "SELECT * FROM fashion_product.t_designer"
        with self.platform_connection.cursor() as cursor:
            cursor.execute(sql)
            for designer in cursor.fetchall():
                self.designers[designer["name"].strip().upper()] = designer
        print json.dumps(self.designers)

    # 加载爬虫商品列表
    def load_products(self):
        sql = """
        SELECT * FROM test.t_intra_mirror a
        """
        with self.intramirror_connection.cursor() as cursor:
            cursor.execute(sql)
            return cursor.fetchall()

    # 通过品牌名称获取品牌信息
    def get_designer_id(self, name):
        name = name.upper()
        if self.designers.get(name):
            return self.designers[name]["designer_id"]

        sql = """
        SELECT * FROM fashion_product.t_designer WHERE `name`=%s
        """
        with self.platform_connection.cursor() as cursor:
            cursor.execute(sql, name)
            designer = cursor.fetchone()
            return designer["designer_id"] if designer else None

    def start(self, shop_id):
        for product in self.load_products():
            product["_name"] = product["name"]
            product["_status"] = "Created"
            product["shop_id"] = shop_id
            product["currency"] = "EUR"
            product["designer_id"] = self.get_designer_id(product["designer"])
            product["sn"] = "intramirror:" + str(product["product_id"])
            # 创建商品
            self.create_product(product)

    # 检查商品是否存在
    def check_product(self, product):
        sql = """
        SELECT * FROM fashion_product.t_shop_product WHERE sn=%s LIMIT 1
        """
        with self.platform_connection.cursor() as cursor:
            cursor.execute(sql, product["sn"])
            return cursor.fetchone()

    # 获取爬虫商品sku
    def get_product_skus(self, product):
        sql = """
        SELECT * FROM test.t_intra_mirror_skus WHERE product_id=%s
        """
        with self.intramirror_connection.cursor() as cursor:
            cursor.execute(sql, product["product_id"])
            return cursor.fetchall()

    # 创建商品sku
    def create_product_sku(self, sku, product):
        print "create product sku:", sku
        sql = """
        INSERT INTO `fashion_product`.`t_shop_product_sku` 
        (`shop_product_id`, `size`, `price`, `settlement_price`, `sn`, `status`, `create_time`, `deleted`) 
        VALUES 
        (%(shop_product_id)s, %(size)s, %(price)s, %(sale_price)s, %(im_code)s, 'Created', unix_timestamp(), '0');
        """
        timestamp = str(int(round(time.time() * 1000)))
        sku["shop_product_id"] = product["shop_product_id"]
        sku["im_code"] = "intramirror#" + (sku["code"] if sku["code"] != "#" else timestamp)
        sku["size"] = sku["size"]
        with self.platform_connection.cursor() as cursor:
            cursor.execute(sql, sku)
            sku["shop_product_sku_id"] = cursor.lastrowid
            self.platform_connection.commit()

    # 设置库存数量
    def create_product_stock(self, sku, product):
        print "create product sku stock:", sku["quantity"], ">>>>>>>>>>>>>>>"
        sql = """
        REPLACE INTO `fashion_product`.`t_shop_stock` 
        (`shop_product_id`, `shop_product_sku_id`, `shop_id`, `quantity`, `create_time`) 
        VALUES 
        (%(shop_product_id)s,%(shop_product_sku_id)s,%(shop_id)s,%(quantity)s,unix_timestamp());
        """
        sku["shop_id"] = product["shop_id"]
        with self.platform_connection.cursor() as cursor:
            cursor.execute(sql, sku)
            self.platform_connection.commit()

    def update_product(self, product):
        print "[%d]product exists, skip >>>>>>>>>>" % self.updated_counter, product["product_id"]
        return

    def get_season_code(self, product):
        referer = json.loads(product["referer"])
        return referer["season_code"] if referer.get("season_code") else None

    # 从 intramirror 爬虫库创建商品到平台库
    def create_product(self, product):
        # 检查商品是否存在
        product_exists = self.check_product(product)
        if product_exists:
            self.updated_counter = self.updated_counter + 1
            return self.update_product(product)

        product["season_code"] = self.get_season_code(product)
        print "[%d]create product:" % self.created_counter, product
        sql = """
        INSERT INTO `fashion_product`.`t_shop_product` 
        (`name`, `sn`, `price`, `settlement_price`, `currency`, `designer_id`,`shop_id`, `status`, `cover`, `images`,`season`,`create_time`) 
        VALUES 
        (%(_name)s,%(sn)s,%(min_price)s,NULL,%(currency)s,%(designer_id)s,%(shop_id)s,%(_status)s,%(cover)s,%(image_urls)s,%(season_code)s,unix_timestamp());
        """
        with self.platform_connection.cursor() as cursor:
            cursor.execute(sql, product)
            product["shop_product_id"] = cursor.lastrowid
            self.platform_connection.commit()
            self.created_counter = self.created_counter + 1

        for sku in self.get_product_skus(product):
            self.create_product_sku(sku, product)
            if sku["quantity"]:
                self.create_product_stock(sku, product)


if __name__ == "__main__":
    # DesignerSynchronizer().start()
    ProductSynchronizer().start(8)



# -*- coding: UTF-8 -*-
from modules import network
import json


def get_product_gender(product):
    if 1568 == product["category1Id"]:
        return 1
    elif 1499 == product["category1Id"]:
        return 2
    else:
        return 3


def get_product_categories(product):
    categories_string = "%s/%s" % (product["category1"], product["category2"])
    return categories_string.split("/")


def get_product_sizes(product):
    skus = product["skus"] if product.get("skus") else []
    supply_price = product.get("salePrice")
    original_price = product.get("price")
    return [
        {
            "size": sku["size"],
            "quantity": sku["count"],
            "supply_price": supply_price,
            "original_price": original_price,
            "currency": "EURO",
            "label": sku["count"]
        }
        for sku in skus
    ]


def get_product_season(catalog):
    context = catalog.get('context')
    if context and context.get("context"):
        context = json.loads(context.get("context"))
        return context.get("season")


def crawl_intramirror_product(catalog):
    page_content = network.download(catalog["url"])
    product = json.loads(page_content.decode(encoding="utf-8"))
    if not product.get("productStatus"):
        print("product no found:", product, catalog["url"])
        return None

    # return this.sourceProduct.category1 + "/" + this.sourceProduct.category2
    # print (json.dumps(product).decode("unicode_escape"))
    return {
        "name": product["name"],
        "source_id": catalog["source_id"],
        "source_type": catalog["source_type"],
        "image_urls": product["pics"],
        "description": product["description"],
        "designer_name": product["brand"],
        "designer_code": product["brandCode"],
        "season": get_product_season(catalog),
        "gender": get_product_gender(product),
        "categories": get_product_categories(product),
        "sizes": get_product_sizes(product),
        "context": product
    }



from task import create_haozhuo_spider_task

create_haozhuo_spider_task(
    url="https://www.farfetch.cn/it/shopping/women/michel-klein-maglioncino-con-fiocco-item-12222186.aspx?storeid=9560&from=listing",
    url_id=0, flag="005", spider_product_id=38979)



from mozia.crawler.monitor import start_product_monitor

from modules.repository import repositories
from modules.scheduler import task_scheduler

if __name__ == '__main__':
    repositories.connect(host='10.26.235.6', port=3066)
    task_scheduler.connect()
    start_product_monitor()
    # app.run(debug=True, port=9604, host="0.0.0.0")



import pymysql


class Repositories:
    def __init__(self):
        self.host = "172.16.8.147"
        self.port = 3306
        self.user = "dba"
        self.passwd = "123456"
        self.db = "farfetch"
        self.charset = 'utf8mb4'
        self.cursorclass = pymysql.cursors.DictCursor,
        self.repositories = []

    def connect(self, host=None,
                port=None,
                user=None,
                passwd=None,
                db=None):
        self.host = host or self.host
        self.port = port or self.port
        self.user = user or self.user
        self.passwd = passwd or self.passwd
        self.db = db or self.db

        for repository in self.repositories:
            repository.connect()

    def add(self, repository):
        print "repository:", repository.__class__
        self.repositories.append(repository)


repositories = Repositories()


class Repository:
    def __init__(self,
                 host=None,
                 port=None,
                 user=None,
                 passwd=None,
                 db=None,
                 cursorclass=pymysql.cursors.DictCursor,
                 charset='utf8mb4'):
        self.host = host
        self.port = port
        self.user = user
        self.passwd = passwd
        self.db = db
        self.connection = None
        self.charset = charset
        self.cursorclass = cursorclass
        repositories.add(self)

    def connect(self):
        if self.connection:
            self.connection.close()
            self.connection = None

        db = self.db or repositories.db
        user = self.user or repositories.user
        host = self.host or repositories.host
        port = self.port or repositories.port
        cursorclass = self.cursorclass or repositories.cursorclass
        passwd = self.passwd or repositories.passwd
        charset = self.charset or repositories.charset

        print "connect to %s@%s:%s - %s" % (user, host, port, db)

        self.connection = pymysql.connect(host=host,
                                          port=port,
                                          user=user,
                                          passwd=passwd,
                                          db=db,
                                          cursorclass=cursorclass,
                                          charset=charset)



# -*- coding: UTF-8 -*-
import json

from keys import KEY_CATALOG_TASK, KEY_PRODUCT_TASK, KEY_HAOZHUO_PRODUCT_TASKS
from modules.scheduler import task_scheduler
from modules.tools import DatetimeEncoder
from repository import dao


def create_product_tasks():
    if task_scheduler.redis.llen(KEY_PRODUCT_TASK) > 0:
        return

    dao.reset()
    for task in dao.catalog.get_product_tasks():
        print "create product tasks:", task["url"]
        task_scheduler.push(KEY_PRODUCT_TASK, json.dumps(task, cls=DatetimeEncoder))


def create_catalog_tasks():
    if task_scheduler.redis.llen(KEY_CATALOG_TASK) > 0:
        return

    dao.reset()
    for catalog_task in dao.common.get_catalog_tasks():
        print "[%s][%s]create catalog task:" % (catalog_task["task_id"], catalog_task["gender"]), catalog_task["url"]
        task_scheduler.push(KEY_CATALOG_TASK, json.dumps(catalog_task, cls=DatetimeEncoder))


def create_description_zh_tasks():
    for task in dao.product.get_product_descriptions_zh_need_translate():
        url = task["source_url"]
        if url.startswith("https://www.farfetch.cn/cn"):
            url = url.replace("https://www.farfetch.cn/cn", "https://www.farfetch.cn/it")
            zh_task = {
                "url": url,
                "flag": task["flag"],
                "url_id": 0,
                "spider_product_id": task["spider_product_id"]
            }
            print "create description zh task:", zh_task
            task_scheduler.push(KEY_HAOZHUO_PRODUCT_TASKS, json.dumps(zh_task))


def create_haozhuo_spider_task(url, flag, url_id, spider_product_id):
    zh_task = {
        "url": url,
        "flag": flag,
        "url_id": url_id,
        "spider_product_id": spider_product_id
    }
    print "create haozhuo description task:", zh_task
    task_scheduler.push(KEY_HAOZHUO_PRODUCT_TASKS, json.dumps(zh_task))


if __name__ == "__main__":
    create_product_tasks()
    create_catalog_tasks()



# -*- coding: UTF-8 -*-
import json
import re
import time
import urllib

from bs4 import BeautifulSoup

from keys import KEY_CATALOG_TASK
from modules.scheduler import task_scheduler
from repository import dao
from task import create_catalog_tasks


class Catalog:
    def __init__(self, language_id=1):
        self.language_id = language_id
        self.store_url = None

    def get_language_id(self):
        return self.language_id

    # 检查商品是否发生了变化
    @staticmethod
    def has_change(product, product_exists):
        context = product["context"]
        context = sorted(context.items(), lambda x, y: cmp(x[1], y[1]))
        context_exists = json.loads(product_exists["context"])
        context_exists = sorted(context_exists.items(), lambda x, y: cmp(x[1], y[1]))
        return str(context) == str(context_exists)

    @staticmethod
    def get_total_page(soup):
        pagination = soup.find("li", class_="pagination-label")
        total_page = pagination.find("span", attrs={
            "data-tstid": "paginationTotal"
        }).string
        return int(total_page)

    def create_catalogs(self, soup):
        soup = soup.find(text=re.compile("window.universal_variable.listing"))
        products = json.loads(soup.string.strip().replace("window.universal_variable.listing =", "").rstrip(";"))
        category_id = products["categoryId"] if products.get("categoryId") else None

        for item in products["items"]:
            product = {
                "catalog_context": json.dumps(item),
                "product_name": item[u"name"],
                "cover": item[u"imageUrl"],
                "url": item[u"url"],
                "store_id": item[u"storeId"],
                "stock": item[u"stock"],
                "designer": item[u"designerName"],
                "currency": item[u"currency"],
                "price": item[u"unit_price"],
                "product_id": item[u"id"],
                "category_id": category_id,
                "sizes": None,
                "tag": None,
                "language_id": self.get_language_id()
            }
            yield dict(item, **product)

    def crawl(self, page_url, task):
        print "begin url:", page_url
        page = urllib.urlopen(page_url)
        html = page.read()
        soup = BeautifulSoup(html, "html.parser")
        for catalog in self.create_catalogs(soup):
            catalog["task_id"] = task["task_id"]
            catalog["gender"] = task["gender"]
            dao.catalog.save_catalog(catalog)
        print "end url:", page_url
        dao.common.update_catalog_task(task)
        return soup

    def crawl_pages(self, task):
        try:
            language = "cn" if 1 == self.get_language_id() else "it"
            url = task[u"url"].strip().replace("/it/", "/%s/" % language)
            self.store_url = url
            soup = self.crawl(url, task)
            quo = url.find("?")
            total_page = self.get_total_page(soup)
            print "total page(%d):" % total_page, url
            for i in range(2, total_page):
                page_url = "%s&page=%d" % (url, i) if quo > 0 else "%s?page=%d" % (url, i)
                self.crawl(page_url, task)
        except Exception as e:
            print 'repr(e):\t', repr(e)

    def start(self):
        catalog_task = task_scheduler.pop(KEY_CATALOG_TASK)
        while catalog_task:
            timestamp = int(time.time())
            catalog_task = json.loads(catalog_task)
            print "[%s]start crawl catalog:" % catalog_task["task_id"], catalog_task["url"]
            self.crawl_pages(catalog_task)
            print "[%s]end crawl catalog, cost(%d)s" % (catalog_task["task_id"], int(time.time()) - timestamp)
            catalog_task = task_scheduler.pop(KEY_CATALOG_TASK)

    @staticmethod
    def get_catalog(product_id):
        return dao.catalog.get_catalog_by_product_id(product_id)


if __name__ == "__main__":
    while True:
        create_catalog_tasks()
        Catalog(language_id=1).start()
        Catalog(language_id=2).start()
        print ">>>>>>>>>>> [S]create catalog tasks and sleep <<<<<<<<<<<"
        time.sleep(100)



KEY_PRODUCT_TASK = "farfetch.product.urls"
KEY_CATALOG_TASK = "farfetch.catalogs.urls"
KEY_PRODUCT_OVERSOLD = "farfetch.product.oversold"
KEY_DOWNLOAD_IMAGES = "farfetch.download.images"
KEY_MONITOR_PRODUCT_TASKS = "farfetch.monitor.product.tasks"
KEY_HAOZHUO_PRODUCT_TASKS = "farfetch:haozhuo:product:tasks"



# -*- coding: UTF-8 -*-
# 读取docx中的文本代码示例
import sys

import docx

from modules.scheduler.scheudler import RedisTaskScheduler
from repository import dao

reload(sys)  # Python2.5 初始化后删除了 sys.setdefaultencoding 方法，我们需要重新载入
sys.setdefaultencoding('utf-8')

task_scheduler = RedisTaskScheduler()

KEY_COVER_PRODUCTS_APPOINT = "farfetch:cover:product:appoint"

farfetch_urls = []
farfetch_codes = []


def create_urls_from_file(file_path):
    # 获取文档对象
    file = docx.Document(file_path)
    print("段落数:" + str(len(file.paragraphs)))  # 段落数为13，每个回车隔离一段

    # 输出每一段的内容
    for para in file.paragraphs:
        text = para.text
        if text.startswith("https"):
            farfetch_urls.append(text.strip())


# # 输出段落编号及段落内容
# for i in range(len(file.paragraphs)):
#     print("第" + str(i) + "段的内容是：" + file.paragraphs[i].text)


if __name__ == '__main__':
    file_path = "data/xiaobao/julianfashion1.docx"
    create_urls_from_file(file_path)
    file_path = "data/xiaobao/wise1.docx"
    create_urls_from_file(file_path)

    for url in farfetch_urls:
        splices = url.split("-")
        last_splice = splices[-1]
        splices2 = last_splice.split(".")
        print splices2[0], "<--->", url
        farfetch_codes.append(splices2[0])

    print "total codes length:", len(farfetch_codes)
    print ",".join(farfetch_codes)

    for item in dao.common.get_appoint_products():
        task_scheduler.push(KEY_COVER_PRODUCTS_APPOINT, item["spider_product_id"])



# -*- coding: UTF-8 -*-
import json
import time
from datetime import date, datetime
import decimal
from repository import dao
from modules.repository import repositories
import pymysql


class DatetimeEncoder(json.JSONEncoder):
    def default(self, obj):
        # if isinstance(obj, datetime.datetime):
        #     return int(mktime(obj.timetuple()))
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')
        elif isinstance(obj, date):
            return obj.strftime('%Y-%m-%d')
        elif isinstance(obj, decimal.Decimal):
            return float(obj)
        else:
            return json.JSONEncoder.default(self, obj)


class SpiderProductTransmitter:
    def __init__(self):
        self.filter_repository = dao.filter
        self.source_connection = pymysql.connect(host='172.16.8.147',
                                                 port=3306,
                                                 user='dba',
                                                 passwd='123456',
                                                 db="spider2",
                                                 cursorclass=pymysql.cursors.DictCursor,
                                                 charset='utf8mb4')

        self.target_connection = pymysql.connect(host='10.26.235.6',
                                                 port=3066,
                                                 user='dba',
                                                 passwd='123456',
                                                 db="fashion_product",
                                                 cursorclass=pymysql.cursors.DictCursor,
                                                 charset='utf8mb4')
        # ProductionConnection("fashion_product").connection
        time_string = time.strftime('%Y%m%d%H%M%S', time.localtime(time.time()))
        self.file_error = open("data/product_%s_error.txt" % time_string, "w")
        self.file_copied = open("data/product_%s_copy.txt" % time_string, "w")
        self.file_exists = open("data/product_%s_exists.txt" % time_string, "w")
        self.create_product_number = 0
        self.create_product_images_number = 0
        self.create_product_sku_number = 0
        self.create_product_description_number = 0

    def __del__(self):
        self.file_error.close()
        self.file_copied.close()
        self.file_exists.close()
        self.source_connection.close()
        self.target_connection.close()

    def check_target_product(self, spider_product_id):
        sql = """
        SELECT * FROM t_spider_product WHERE spider_product_id=%s
        """
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql, spider_product_id)
            return cursor.fetchone()

    def get_source_product(self, spider_product_id):
        sql = """
        SELECT * FROM spider2.t_spider_product WHERE spider_product_id=%s
        """
        with self.source_connection.cursor() as cursor:
            cursor.execute(sql, spider_product_id)
            return cursor.fetchone()

    def get_source_product_description(self, spider_product_id):
        sql = """
        SELECT * FROM spider2.t_spider_product_description WHERE spider_product_id=%s
        """
        with self.source_connection.cursor() as cursor:
            cursor.execute(sql, spider_product_id)
            return cursor.fetchall()

    def get_source_product_images(self, spider_product_id):
        sql = """
        SELECT * FROM spider2.t_spider_product_images WHERE spider_product_id=%s
        """
        with self.source_connection.cursor() as cursor:
            cursor.execute(sql, spider_product_id)
            return cursor.fetchone()

    def get_source_product_skus(self, spider_product_id):
        sql = """
        SELECT   * FROM  spider2.t_spider_product_sku  WHERE  spider_product_id=%s
        """
        with self.source_connection.cursor() as cursor:
            cursor.execute(sql, spider_product_id)
            return cursor.fetchall()

    def commit_target_product(self, source_product, descriptions, product_images,
                              product_skus):
        self.create_target_product(source_product)
        self.create_product_descriptions(descriptions)
        self.create_product_images(product_images)
        self.create_product_skus(product_skus)

        self.target_connection.commit()

        self.create_product_number = self.create_product_number + 1
        self.create_product_images_number = self.create_product_images_number + 1
        self.create_product_sku_number = self.create_product_sku_number + len(product_skus)
        self.create_product_description_number = self.create_product_description_number + len(descriptions)

    def create_target_product(self, source_product):
        sql = """
        INSERT INTO `fashion_product`.`t_spider_product` 
        (`spider_product_id`, `designer`, `gender`, `category`, `cover`, `status`, `flag`, `color_code`, 
        `designer_code`, `resource_code`, `season`, `source_url`, `editor_name`, `date_added`, `selling_status`) 
        VALUES 
        (%(spider_product_id)s, %(designer)s, %(gender)s, %(category)s, %(cover)s, 1, %(flag)s, %(color_code)s, 
        %(designer_code)s, %(resource_code)s, %(season)s, %(source_url)s, "yuanjie", now(), %(selling_status)s);
        """
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql, source_product)
            # self.target_connection.commit()

    def create_product_descriptions(self, source_descriptions):
        sql = """
        INSERT INTO `fashion_product`.`t_spider_product_description` 
        (`spider_product_id`, `name`, `description`, `constitute`, `location`, `size_description`, 
        `category_description`, `language_id`, `editor_name`, `date_added`) VALUES 
        (%(spider_product_id)s, %(meta_name)s, %(description)s, %(constitute)s, %(location)s, %(size_description)s,
        %(category_description)s, %(language_id)s, "yuanjie", now());
        """
        with self.target_connection.cursor() as cursor:
            for description in source_descriptions:
                description["meta_name"] = description["name"]
                cursor.execute(sql, description)
                print "save product description:", description
                # self.target_connection.commit()

    def create_product_images(self, product_images):
        sql = """
        INSERT INTO `fashion_product`.`t_spider_product_images` 
        (`spider_product_id`, `images`, `farfetch_images`, `date_added`) VALUES 
        (%(spider_product_id)s, %(images)s, %(farfetch_images)s, now());
        """
        with self.target_connection.cursor() as cursor:
            cursor.execute(sql, product_images)
            # self.target_connection.commit()

    def create_product_skus(self, products_skus):
        sql = """
        INSERT INTO `fashion_product`.`t_spider_product_sku` 
        (`spider_product_sku_id`, `spider_product_id`, `size`, `price`, `discount_price`, `currency`, `editor_name`, 
        `date_added`, `stock_status`) 
        VALUES 
        (%(spider_product_sku_id)s, %(spider_product_id)s, %(size)s, %(price)s, %(discount_price)s, %(currency)s, "yuanjie", 
        now(), %(stock_status)s);
        """
        with self.target_connection.cursor() as cursor:
            for sku in products_skus:
                cursor.execute(sql, sku)
                print "save product sku:", sku
                # self.target_connection.commit()

    def copy_product_from_source_17fw(self):
        for source in self.filter_repository.find_17fw_products():
            target_product = self.check_target_product(source["spider_product_id"])
            if target_product:
                print "target product exists:", source
                self.file_exists.write(json.dumps(source))
                continue
            print "start create product:", source
            source_product = self.get_source_product(source["spider_product_id"])
            source_product_descriptions = self.get_source_product_description(source["spider_product_id"])
            source_product_images = self.get_source_product_images(source["spider_product_id"])
            source_product_skus = self.get_source_product_skus(source["spider_product_id"])

            # 设置季节
            source_product["season"] = "17FW"

            if source_product and source_product_descriptions and source_product_images and source_product_skus:
                self.commit_target_product(source_product, source_product_descriptions, source_product_images,
                                           source_product_skus)
                source_product["descriptions"] = source_product_descriptions
                source_product["images"] = source_product_images
                source_product["skus"] = source_product_skus
                self.file_copied.write(json.dumps(source_product, cls=DatetimeEncoder))
            else:
                print "product invalid:", source
                self.file_error.write(json.dumps(source, cls=DatetimeEncoder))

        print "copy products finish:", self.create_product_number, self.create_product_images_number, \
            self.create_product_description_number, self.create_product_sku_number

    def copy_product_from_source_18ss(self):
        for source in self.filter_repository.find_18ss_products():
            target_product = self.check_target_product(source["spider_product_id"])
            if target_product:
                print "target product exists:", source
                self.file_exists.write(json.dumps(source))
                continue
            print "start create product:", source
            source_product = self.get_source_product(source["spider_product_id"])
            source_product_descriptions = self.get_source_product_description(source["spider_product_id"])
            source_product_images = self.get_source_product_images(source["spider_product_id"])
            source_product_skus = self.get_source_product_skus(source["spider_product_id"])

            # 设置季节
            source_product["season"] = "18SS"

            if source_product and source_product_descriptions and source_product_images and source_product_skus:
                self.commit_target_product(source_product, source_product_descriptions, source_product_images,
                                           source_product_skus)
                source_product["descriptions"] = source_product_descriptions
                source_product["images"] = source_product_images
                source_product["skus"] = source_product_skus
                self.file_copied.write(json.dumps(source_product, cls=DatetimeEncoder))
            else:
                print "product invalid:", source
                self.file_error.write(json.dumps(source, cls=DatetimeEncoder))

        print "copy products finish:", self.create_product_number, self.create_product_images_number, \
            self.create_product_description_number, self.create_product_sku_number


if __name__ == '__main__':
    repositories.connect()
    SpiderProductTransmitter().copy_product_from_source_17fw()
    SpiderProductTransmitter().copy_product_from_source_18ss()



# -*- coding: UTF-8 -*-
import json
import re
import time

from bs4 import BeautifulSoup

import network
from converter import ProductConverter
from keys import KEY_PRODUCT_TASK, KEY_DOWNLOAD_IMAGES, KEY_PRODUCT_OVERSOLD
from modules.scheduler import task_scheduler
from repository import dao
from task import create_product_tasks


class Product:
    def __init__(self):
        self.host = "https://www.farfetch.cn/"

    @staticmethod
    def get_language(language_id):
        if 1 == language_id:
            return "cn"
        elif 3 == language_id:
            return "it"
        elif 2 == language_id:
            return "it"

    def get_sizes_url(self, product, language_id):
        url = "/product/GetDetailState?productId=%s&storeId=%s&sizeId=&categoryId=%s&designerId=%s"
        return self.host + self.get_language(language_id) + url % (product["product_id"],
                                                                   product["store_id"],
                                                                   product["category_id"],
                                                                   product["manufacturer_id"])

    @staticmethod
    def get_store_id(product, url):
        if product.get("storeId"):
            return product["storeId"]

        pattern = re.compile("storeid=(\d+)")
        match = pattern.findall(url)
        if match:
            return match[0]
        return None

    def crawl(self, url, index, link):
        timestamp = int(time.time())
        print("begin url:%s - %s, %s" % (url, index, time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))))
        html = network.download(url)
        if not html:
            return

        soup = BeautifulSoup(html, "html.parser")
        script_soup = soup.find("script", text=re.compile("window.universal_variable.product\s*=\s*"))

        # 找不到商品信息
        if not script_soup:
            # 商品已售罄
            print "product sold out:", link
            dao.catalog.set_catalog_status(link, 2)
            task_scheduler.push(KEY_PRODUCT_OVERSOLD, json.dumps(link))
            return

        product = json.loads(
            script_soup.string.strip().replace("window.universal_variable.product =", "").strip().rstrip(";"))
        product["product_id"] = product["id"]
        product["manufacturer_id"] = product["manufacturerId"]
        product["category_id"] = product["categoryId"]
        product["language_id"] = link["language_id"]
        product["store_id"] = self.get_store_id(product, url)
        product["source_url"] = url

        # 获取尺码的请求地址
        sizes_url = self.get_sizes_url(product, link["language_id"])

        # 获取尺码信息
        sizes_string = network.download(sizes_url)
        # 将内容提交给解析器
        converter = ProductConverter(soup, product, sizes_string, link["language_id"])
        # 开始解析商品
        product = converter.create()

        print "product:", json.dumps(product)
        # 没有尺码 / 售罄
        if converter.is_sold_out() or not converter.get_product_sizes():
            print "product sold out", product["product_id"]
            product["product_status"] = "OVERSOLD"

        # 保存到数据库
        dao.product.save(product, link["language_id"])

        # # 添加图片下载任务
        task_scheduler.push(KEY_DOWNLOAD_IMAGES, json.dumps({
            "product_id": product["product_id"],
            "image_urls": product["image_urls"]
        }))

        # 更新目录状态
        dao.catalog.set_catalog_status(link, 1)
        print("end url:%s - %s, cost:%d" % (url, index, int(time.time()) - timestamp))

    def start(self):
        index = 0
        product_task = task_scheduler.pop(KEY_PRODUCT_TASK)
        while product_task:
            timestamp = int(time.time())
            product_task = json.loads(product_task)
            if not product_task.get("language_id"):
                print "product no language special:", product_task["product_id"]
                continue

            print "start crawl product(%d)" % product_task["product_id"]
            print "start task:", product_task
            url = "https://www.farfetch.cn%s" % product_task["url"]
            self.crawl(url, index, product_task)
            index = index + 1
            print "end crawl product(%d), cost(%d)s" % (product_task["product_id"], int(time.time()) - timestamp)
            product_task = task_scheduler.pop(KEY_PRODUCT_TASK)


if __name__ == "__main__":
    while True:
        create_product_tasks()
        Product().start()
        print ">>>>>>>>>>> [S]create product tasks and sleep <<<<<<<<<<<"
        time.sleep(30)



import time

from modules.scheduler.scheudler import RedisTaskScheduler
from modules.tools import translator, contain_zh
from repository import dao
from task import get_translate_task, create_translate_tasks

task_scheduler = RedisTaskScheduler()


def start_translate():
    task = get_translate_task()
    while task:
        composition = task["composition"]
        if not composition:
            continue

        composition_zh = translator.en_to_zh(composition)
        print "translate composition ok:", composition_zh, task["product_id"]
        task["composition"] = composition_zh
        task["language_id"] = 1

        # if task["name"] and not contain_zh(task["name"]):
        #     task["product_name"] = translator.en_to_zh(task["name"])
        #     print "translate name ok:", task["product_name"], task["name"]
        # else:
        task["product_name"] = task["name"]

        description = task["description"]
        if description and not contain_zh(description):
            task["description"] = translator.en_to_zh(description)
            print "translate description ok:", task["description"]

        location = task["location"]
        if location and not contain_zh(location):
            task["location"] = translator.en_to_zh(location)
            print "translate location ok:", task["location"]

        dao.product.save_description(task)
        task = get_translate_task()


if __name__ == '__main__':
    while True:
        timestamp = int(time.time())
        print ">>>>>>begin translate", time.time()
        try:
            start_translate()
            create_translate_tasks()
        except Exception as e:
            print "translate error:", e
        print ">>>>>>end translate, cost:", int(time.time()) - timestamp
        time.sleep(30)
